@@ -1,3 +1,6 @@
+"""
+Template and interface classes that handle evaluator inputs and outputs.
+"""
 from abc import ABC, abstractmethod
 from typing import Union
 
@@ -58,15 +61,22 @@ def get_name(x: Union[str, int, IOBase]):
 
 
 class Descriptor(ReprMixin, ABC):
+    """Base class for descriptors.
+
+    Descriptors describe a set of possible values for a single input.
+    They are used in :class:`Parameters <parameters.Parameter>` to specify
+    the possible input space.
+    """
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.platypus_type = NotImplemented
 
     @abstractmethod
     def validate(self, value) -> bool:
-        """Checks if value is a valid value for this parameter.
+        """Checks if `value` is a valid value for this Descriptor.
 
-        :param value:
+        :param value: The value to check.
         :return: True if the value is valid False otherwise
         """
 
@@ -78,13 +88,20 @@ class Descriptor(ReprMixin, ABC):
 
 
 class AnyValue(Descriptor):
-    """A descriptor that can take on any possible value.
+    """A :class:`Descriptor` that can take on any possible value.
     Intended to be used as a placeholder."""
 
-    def validate(self, value):
+    def validate(self, value) -> bool:
+        """All values are accepted by this :class:`Descriptor`,
+        so validate always returns True
+
+        :param value: The value to validate
+        :return: True
+        """
         return True
 
     def sample(self, value):
+        """This is a placeholder descriptor. Therefore, sampling is not possible."""
         raise NotImplementedError
 
     def __bool__(self):
@@ -94,14 +111,25 @@ class AnyValue(Descriptor):
 
 
 class Selector(ReprMixin, ABC):
-    """Base Class for Selectors, which describe what attribute of the building is read or modified"""
+    """Base Class for Selectors, which describe what attribute of the model is read or modified"""
 
     @abstractmethod
     def get(self, building):
+        """Get the current value of this attribute of the building
+
+        :param building:
+        :return:
+        """
         pass
 
     @abstractmethod
     def set(self, building, value):
+        """Modifies `building`, setting the attribute this selector corresponds to to `value`
+
+        :param building: the building to modify.
+        :param value: the value to set this selector's attribute to.
+        :return:
+        """
         pass
 
     def setup(self, building) -> None:
@@ -114,9 +142,14 @@ class DummySelector(Selector):
     Intended to be used as a placeholder."""
 
     def get(self, building):
+        """Getting the value of this placeholder is not possible.
+
+        This function will raise a NotImplementedError.
+        """
         raise NotImplementedError
 
     def set(self, building, value):
+        """This placeholder will not do anything with the given value."""
         pass
 
     def __bool__(self):
