@@ -1,6 +1,6 @@
 """
-This file is a collection of functions to interact with eppy and geomeppy.
-They could do with some sorting and refactoring.
+Functions to interact with eppy and geomeppy.
+Some functions also support equivalent operations for json buildings.
 """
 
 import contextlib
@@ -13,7 +13,9 @@ import string
 import warnings
 from functools import partial
 from pathlib import Path
+from typing import Optional
 
+import eppy
 import numpy as np
 import pandas as pd
 from eppy.bunch_subclass import BadEPFieldError
@@ -26,6 +28,7 @@ from errors import ModeError
 
 
 def view_building(building):
+    """Displays a matplotlib 3d plot of the building"""
     if get_mode(building) == "json":
         return view_epJSON(building)
     elif get_mode(building) == "idf":
@@ -33,8 +36,7 @@ def view_building(building):
 
 
 def merge_building(building_from, building_to):
-    """Copy objects in building_from to building_to.
-    """
+    """Copy objects in building_from to building_to."""
     mode_from = get_mode(building_from)
     mode_to = get_mode(building_to)
     if mode_to == "idf" and mode_from == "idf":
@@ -74,7 +76,12 @@ def convert_format(s: str, place, mode):
     raise ModeError(message=f"no format defined for place:{place} and mode:{mode}")
 
 
-def get_mode(building):
+def get_mode(building) -> str:
+    """Determines whether building uses an idf or json format
+
+    :param building: a building, either an IDF or a dictionary of json data
+    :return: "idf" or "json"
+    """
     if isinstance(building, IDF):
         return "idf"
     if isinstance(building, dict):
@@ -123,8 +130,7 @@ def get_building(
     version=config.energy_plus_version,
     ep_path=None,
 ):
-    """Get building from directory
-    """
+    """Get building from directory"""
     if mode is None and building is None:
         building = config.files.get("building")
         mode = config.energy_plus_mode
@@ -151,17 +157,17 @@ def get_building(
     ModeError(mode)
 
 
-def get_idfobject_from_name(idf: IDF, name: str):
-    """	get_idfobject_from_name(IDF, str)
-
-    Gets an object from the passed idf where it's name
+def get_idfobject_from_name(
+    idf: IDF, name: str
+) -> Optional[eppy.bunch_subclass.EpBunch]:
+    """Gets an object from the passed idf where it's name
     value is equal to the passed string, if none are found
     then this method returns None
 
-    Args:
-        idf(IDF): The idf to find the object from.
-        name(str): The string to find that is equal to the
-               name field of the object.
+
+    :param idf: The idf to find the object from.
+    :param name: The string to find that is equal to the name field of the object.
+    :return: the object from the idf with the matching name.
     """
 
     #: Scan all objects in all lists
@@ -212,8 +218,7 @@ def wwr_all(building, wwr: float, direction=None) -> None:
 
 
 def get_window_range(windows, building, mode):
-    """ get max and min coordinates range of windows
-    """
+    """get max and min coordinates range of windows"""
     values = {"max_x": -999, "min_x": 999, "max_y": -999, "min_y": 999}
     for window_name, window in windows:
         # getting coordinates here
@@ -234,8 +239,7 @@ def get_window_range(windows, building, mode):
 
 
 def get_coordinates(window, building, mode):
-    """ Get coordinates of the xs, ys, and zs
-    """
+    """Get coordinates of the xs, ys, and zs"""
     if mode == "idf":
 
         def coordinates(ax):
@@ -257,7 +261,7 @@ def get_coordinates(window, building, mode):
 
 def set_vertex(idfObj, vertexNum: int, x: float = 0, y: float = 0, z: float = 0):
     """Sets a single vertex of the passed idfObject (idfObj)
-     to the specified x,y and z coordinates."""
+    to the specified x,y and z coordinates."""
     for val, name in zip((x, y, z), "XYZ"):
         idfObj["Vertex_{}_{}coordinate".format(vertexNum, name)] = round(val, 2)
 
@@ -343,12 +347,12 @@ def check_direct(direction: str, xs, ys, values) -> bool:
 
 # TODO: Document axis
 def set_wwr_single(window, wwr: float, axs, zs, axis):
-    """ Set the single window's wwr
+    """Set the single window's wwr
 
-        :param window: the window to be modified
-        :param wwr: the window to wall ratio of the result
-        :param axs: the axis this window is aligned with
-        :param zs: the z coordinate of the window
+    :param window: the window to be modified
+    :param wwr: the window to wall ratio of the result
+    :param axs: the axis this window is aligned with
+    :param zs: the z coordinate of the window
     """
     width = max(axs) - min(axs)
     scale_factor = np.sqrt(wwr)
@@ -386,7 +390,7 @@ def set_wwr_single(window, wwr: float, axs, zs, axis):
 
 
 def set_daylight_control(building, zone_name, distance, illuminance=500):
-    """ Set daylighting control to the biggest window of the zone.
+    """Set daylighting control to the biggest window of the zone.
 
     :param building: an idf object
     :param zone_name: name of the zone
@@ -512,15 +516,15 @@ def get_reference_point(p1, p2, distance):
 
 
 def read_sql(path: str, cmds: list, direction=None):
-    """ Open sql file with connect
+    """Open sql file with connect
 
-        :param path: absolute directory to the sql file.
-        :param cmds: list of commands need to be processed, 'all' will process all cmds.
-            All commands available : 'all', 'wall area', 'ceiling height', 'floor area', 'volume'
-        :param direction: a list of directions of walls when process wall area cmd,
-            None means taking both area and gross area of walls in all directions and
-            the total areas of all walls.
-        :return: a dictionary of all data desired.
+    :param path: absolute directory to the sql file.
+    :param cmds: list of commands need to be processed, 'all' will process all cmds.
+        All commands available : 'all', 'wall area', 'ceiling height', 'floor area', 'volume'
+    :param direction: a list of directions of walls when process wall area cmd,
+        None means taking both area and gross area of walls in all directions and
+        the total areas of all walls.
+    :return: a dictionary of all data desired.
 
     """
     try:
@@ -551,8 +555,7 @@ def read_sql(path: str, cmds: list, direction=None):
         return azim_range
 
     def wall_area(dire: str):
-        """Find total area with window subtracted from either all walls or only walls in current direction
-        """
+        """Find total area with window subtracted from either all walls or only walls in current direction"""
         table_name = "Surfaces"
         azim_range = get_azim_range(dire)
         if len(azim_range) > 2:
@@ -573,16 +576,14 @@ def read_sql(path: str, cmds: list, direction=None):
         return data
 
     def ceiling_height():
-        """Find height of the ceiling
-        """
+        """Find height of the ceiling"""
         floor = floor_area()
         volume = zone_volume()
         height = volume / floor
         return height
 
     def floor_area():
-        """Find total floor area
-        """
+        """Find total floor area"""
         table_name = "Zones"
         query = f"SELECT SUM(FloorArea) FROM {table_name}"
         cur.execute(query)
@@ -590,8 +591,7 @@ def read_sql(path: str, cmds: list, direction=None):
         return data[0][0]
 
     def zone_volume():
-        """Find total volume of Zones
-        """
+        """Find total volume of Zones"""
         table_name = "Zones"
         query = f"SELECT SUM(Volume) FROM {table_name}"
         cur.execute(query)
@@ -599,8 +599,7 @@ def read_sql(path: str, cmds: list, direction=None):
         return data[0][0]
 
     def sg_temp():
-        """ find site ground temperature
-        """
+        """find site ground temperature"""
         table_name = "ReportDataDictionary"
         query = f"SELECT ReportDataDictionaryIndex FROM {table_name} WHERE Name = 'Site Ground Temperature'"
         cur.execute(query)
@@ -677,14 +676,17 @@ def read_sql(path: str, cmds: list, direction=None):
 # Add direction
 # Add cmds
 def write_csv(path: str, dest: str, shape=False):
-    """ A handler to call read_sql() function and get data, then put the data into exl file
+    """A handler to call read_sql() function and get data, then put the data into exl file
 
-        :param path: path to read sql file
-        :param dest: destination to put data
-        :param shape: for special use, only works with right file name
+    :param path: path to read sql file
+    :param dest: destination to put data
+    :param shape: for special use, only works with right file name
     """
     # Get data from sql file. Currently retrieves all the data.
-    data = read_sql(path, ["all"],)
+    data = read_sql(
+        path,
+        ["all"],
+    )
 
     # Get number of floors and shape from file name
     filename = Path(dest).name
@@ -722,11 +724,11 @@ def write_csv(path: str, dest: str, shape=False):
 
 
 def get_files(directory: str, filetype: str):
-    """ Get all file names in the directory
+    """Get all file names in the directory
 
-        :param directory: directory to the folder contains all files
-        :param filetype: the desired file's types
-        :return: a list of file names 
+    :param directory: directory to the folder contains all files
+    :param filetype: the desired file's types
+    :return: a list of file names
     """
     file_names = []
     entries = Path(directory)
@@ -742,8 +744,7 @@ def get_files(directory: str, filetype: str):
 
 
 def generate_dir(dest_folder=None):
-    """ func use to generate a directory for besos outputs
-    """
+    """func use to generate a directory for besos outputs"""
     folder = "BESOS_Output"
     if dest_folder is not None:
         folder = dest_folder
@@ -756,16 +757,21 @@ def generate_dir(dest_folder=None):
 
 
 def generate_batch(
-    account: str, time: str, email: str, task_id: int, cpu_per_task=1, mem=1000,
+    account: str,
+    time: str,
+    email: str,
+    task_id: int,
+    cpu_per_task=1,
+    mem=1000,
 ):
-    """ function to write a bash file for running jupyter on computer canada
+    """function to write a bash file for running jupyter on computer canada
 
-        :param account: account used
-        :param time: time for bash job
-        :param email: user email
-        :param task_id: task id to use
-        :param cpu_per_task: number of cpu used to run one task
-        :param mem: memory needs in total
+    :param account: account used
+    :param time: time for bash job
+    :param email: user email
+    :param task_id: task id to use
+    :param cpu_per_task: number of cpu used to run one task
+    :param mem: memory needs in total
     """
     # TODO there might be a calculation relationship between mem, cpu and task_id.
     # If possible give some default value to the arguments
@@ -793,8 +799,8 @@ def generate_batch(
 
 
 def convert_to_json(idf: IDF):
-    """ convert idf file to json
-        the func will create a json file that is converted from the idf
-        can use --convert-only when energyplus version 9.3 is in besos
+    """convert idf file to json
+    the func will create a json file that is converted from the idf
+    can use --convert-only when energyplus version 9.3 is in besos
     """
     os.system(f"energyplus -c {idf}")
