@@ -1,19 +1,36 @@
-from typing import List, Dict, Tuple, Union
-from dataclasses import dataclass
-from abc import ABC, abstractmethod
+"""Objectives generate the output of evaluators.
 
-import os
+For energyplus, outputs are calculated by reading the simulation output files,
+using either MeterReaders or VariableReaders.
+
+The optimisation direction (minimize or maximize) and if the objectives are used to put
+constraints on a solution (such as electricity use being less than a certain amount)
+are both specified by arguments to the Problem containing the objectives.
+
+If the model used does not generate the needed output, the objective will modify the model
+to add the corresponding output. For example, if a building does not include an
+Ouptut:Meter object for Electricity:Facility, one will be added if the corresponding
+:class:`MeterReader` is used with that building.
+"""
 import re
+from abc import ABC, abstractmethod
+from dataclasses import dataclass
 from itertools import chain
-import pandas as pd
+from pathlib import Path
+from typing import List, Dict, Tuple, Union, Callable
+from warnings import warn
 
 import config
-from errors import ModeError
-from IO_Objects import Objective
 import eppy_funcs as ef
+import pandas as pd
+from IO_Objects import Objective
+from errors import ModeError
+from packaging.version import Version
+
+from besos.types import PathLike
 
 
-def get_data_dict_regex():
+def _get_data_dict_regex():
     """Generates a regex expression that can parse a line of the data dictionary in the .mtr file
     Various assumptions are made about the exact composition of names."""
     report_code = r"(?P<report_code>\d+)"  # report code of digits
@@ -22,10 +39,9 @@ def get_data_dict_regex():
     units = r"\[(?P<units>[^\]]*?)\]"  # units are enclosed in square brackets.
     frequency = r"!(?P<frequency>[\w|,]+)"  # frequency preceded by a '!'
     remainder = r"(?:  ?\[(?P<remainder>[^\]]+)\])?"  # can be preceded by 1-2 spaces based on experience
-    data_dict_line = re.compile(
+    return re.compile(
         fr"^{report_code},{num_items},{name} {units} {frequency}{remainder}$"
     )
-    return data_dict_line
 
 
 _values_collection = List[List[float]]
@@ -33,7 +49,7 @@ _values_collection = List[List[float]]
 
 @dataclass
 class EPResults:
-    """Represents the information about a Meter read from a .mtr file."""
+    """Represents the information about a Meter or Variable read from a .eso file."""
 
     name: str
     frequency: str
@@ -55,13 +71,13 @@ class EPResults:
         for i, original in enumerate(items):
             unq_names[original] = unq_names.get(original, []) + [i]
         new_names = items[:]
-        for name, values in unq_names.items():
+        for unq_name, values in unq_names.items():
             if len(values) == 1:
                 continue  # not a duplicate, leave the name alone
             for count, index in enumerate(values):
-                new_names[index] = f"{name}_{count}"
+                new_names[index] = f"{unq_name}_{count}"
         if prefix is not None:
-            names = [f"{prefix}_{name}" for name in new_names]
+            names = [f"{prefix}_{base_name}" for base_name in new_names]
         else:
             names = new_names
         df = pd.DataFrame({name: values for name, values in zip(names, zip(*data))})
@@ -71,29 +87,72 @@ class EPResults:
 _results_format = Dict[Tuple[str, str], EPResults]
 
 
+def get_eso_version(eso: PathLike) -> Version:
+    """Gets the version of energyplus used to generate an eso file.
+
+    :param eso: the path to the eso file
+    :return: the version of energyplus used to generate the eso file.
+    """
+    # a regular expression to extract information from the first line of an eso file
+    first_line_regex = re.compile(
+        r"^Program Version,EnergyPlus, Version "  # constant value at the start of the line
+        r"(?P<version_number>[\d.]+)-(?P<hash>[0-9a-z]{10}), "
+        r"YMD=(?P<date>\d\d\d\d\.\d\d\.\d\d+) (?P<time>\d?\d:\d\d)$",
+    )
+    with open(eso, "r") as f:
+        first_line = f.readline()
+    match = first_line_regex.match(first_line)
+    if match is None:
+        raise ValueError(
+            f"First line of file:'{first_line}' did not match the expected format."
+        )
+    return Version(match.group("version_number"))
+
+
 def read_eso(
-    out_dir: str = config.out_dir,
+    out_dir: PathLike = config.out_dir,
     file_name="eplusout.eso",
-    version=config.energy_plus_version,
+    version=None,
 ) -> _results_format:
     """Retrieve information from an Energy-Plus simulation, with outputs in out_dir
 
-    returns a dictionary with keys of the form {(MeterName, Reporting_Frequency or None):EPResults dataclass}"""
+    :param out_dir: the directory that contains the output files
+    :param file_name: the name of the output file to read from. Should be a .eso file.
+    :param version: Deprecated argument. Do not use.
+    :return: a dictionary with keys of the form {(MeterName, Reporting_Frequency or None):EPResults dataclass}
+    """
     # output format described here:
     # https://energyplus.net/sites/default/files/pdfs_v8.3.0/OutputDetailsAndExamples.pdf
 
-    with open(os.path.join(out_dir, file_name), "r") as file:
+    eso_path = Path(out_dir, file_name)
+    eso_version = get_eso_version(eso_path)
+    with open(eso_path, "r") as file:
         f = file.readlines()  # may cause memory errors on very large files
 
+    # Backwards compatibility warning.
+    if version:
+        warn(
+            "The version number is now auto-detected."
+            "The version argument is deprecated "
+            "and will be removed in the future.",
+            FutureWarning,
+        )
+        if Version(version) != eso_version:
+            raise ValueError(
+                f"Version {version} provided, "
+                f"but eso file is from version {eso_version}"
+            )
+    del version
+
     # TODO: Find a way to avoid hardcoding these values to make things portable between versions
     # or move them to a version-dependant config of some kind
-    data_dictionary_start = 1  # omit the first line, it is E+ version info
-    if float(version[:3]) > 8.8:
+    if eso_version >= Version("9.0"):
         # header is always 5 lines for E+ 8.8 and 6 lines for E+ 9.0
         data_dictionary_header = 6
     else:
-        data_dictionary_header = 5
+        data_dictionary_header = 5  # pragma: nocover
 
+    data_dictionary_start = 1  # first line has already been processed
     data_dictionary_end = f.index("End of Data Dictionary\n")
     data_end = -2  # omit the End of Data line and the Number of Records Written line
 
@@ -101,12 +160,12 @@ def read_eso(
     requested_vars = f[data_dictionary_header + 1 : data_dictionary_end]
     data = f[data_dictionary_end + 1 : data_end]
 
-    data_dict_line = get_data_dict_regex()
+    data_dict_line = _get_data_dict_regex()
     codes = {}
     keys = set()
-    # currently ignores the header
+
     for line in requested_vars:
-        match = re.search(data_dict_line, line)
+        match = data_dict_line.search(line)
         if match is None:
             raise ValueError(f"could not match: {line}")
         name = match.group("name")
@@ -140,23 +199,27 @@ def read_eso(
     }
 
 
-# TODO: Finish
 def time_series_values(results: EPResults):
-    """Returns the entire value column from some results formated as a pandas DF
+    """Returns the entire value column from the results.
 
-    :param values: the values to display
-    :return: the formated DF of the values in the collection.
+    :param results: the results object
+    :return: the values from the results object.
     """
-
-    # Already is a pandas DF [to_dict() can be used to convert it to the timeseries dict now or later in a differnt function?(maybe)]
-    # possibly unit conversions as well? (althoug that should maybe be done in a different function at a differnt time - would be easy if its a pandas dataframe)
+    # TODO: Finish
+    # Already is a pandas DF [to_dict() can be used to convert it to the
+    # timeseries dict now or later in a different function?(maybe)]
+    # possibly unit conversions as well? (although that should maybe
+    # be done in a different function at a different time - would be easy if its a pandas dataframe)
+    warn(
+        "`time_series_values` is incomplete, and returns raw values, not time series values."
+    )
     return results.data["Value"]
 
 
 def sum_values(results: EPResults) -> float:
     """Returns the sum over the Value column from some results
 
-    :param values: the values to sum over
+    :param results: the results object to sum over
     :return: the sum of the first value for each entry in the collection.
     """
     return results.data["Value"].sum()
@@ -164,7 +227,7 @@ def sum_values(results: EPResults) -> float:
 
 # TODO make "all" clear all optional output, currently incomplete (i.e. csv files)
 def clear_outputs(building, outputs: Union[str, List[str]] = "all") -> None:
-    """Disable certain types of output.
+    """Disable certain types of output that appear in the .eso file.
 
     :param building: the building to modify
     :param outputs: Can be the class_name of the output to clear,
@@ -222,10 +285,10 @@ def clear_outputs(building, outputs: Union[str, List[str]] = "all") -> None:
 
 
 class EPReader(Objective, ABC):
-    field_pairs = NotImplemented
+    field_pairs: tuple = NotImplemented
 
-    def __init__(self, class_name, frequency: str = None, func=sum_values, **kwargs):
-        super().__init__(**kwargs)
+    def __init__(self, class_name, frequency: str = None, func=sum_values, name=""):
+        super().__init__(name=name)
         self.class_name = class_name
         self.frequency = frequency
         # class name has a nonzero default, so this may be adding noise to the repr
@@ -256,7 +319,12 @@ class EPReader(Objective, ABC):
         return True
 
     def get_objective(self, building):
+        """Retrieves output object in the building corresponding to this Objective.
 
+        :param building: The building to search for the object.
+        :return: The part of the building's data corresponding to this Objective.
+            The return type varies between json and idf buildings
+        """
         mode = ef.get_mode(building)
 
         if mode == "idf":
@@ -350,14 +418,22 @@ class MeterReader(EPReader):
 
     def __init__(
         self,
-        key_name,
-        class_name=config.objective_meter_type,
+        key_name: str,
+        class_name: str = config.objective_meter_type,
         frequency: str = None,
-        func=sum_values,
-        **kwargs,
+        func: Callable = sum_values,
+        name: str = "",
     ):
+        """
+
+        :param key_name: What to measure (e.g. Electricity:Facility)
+        :param class_name: The kind of meter to use (e.g. Output:Meter)
+        :param frequency: the reporting frequency of the meter. (e.g. Timestep, Monthly)
+        :param func: a function used to aggregate the results read from the file.
+        :param name: the name of the output column corresponding to this objective
+        """
         super().__init__(
-            class_name=class_name, frequency=frequency, func=func, **kwargs
+            class_name=class_name, frequency=frequency, func=func, name=name
         )
         self.key_name = key_name
         self._add_repr("key_name", check=True)
@@ -377,17 +453,29 @@ class VariableReader(EPReader):
         ("frequency", "Reporting_Frequency"),
     )
 
+    # TODO check if class_name must allays be Output:Variable
     def __init__(
         self,
         key_value,
         variable_name="*",
         class_name=config.objective_variable_type,
         frequency: str = None,
-        func=sum_values,
-        **kwargs,
+        func: Callable = sum_values,
+        name="",
     ):
+        """Reads the values of Variables from the .eso file
+        Energyplus docs:
+        https://bigladdersoftware.com/epx/docs/8-9/input-output-reference/input-for-output.html#outputvariable
+
+        :param key_value: What to measure (e.g. Electricity:Facility)
+        :param variable_name:  The kind of variable to use (e.g. Output:Variable)
+        :param class_name: the type of variable to read from.
+        :param frequency: the reporting frequency of the variable.
+        :param func: a function used to aggregate the results read from the file.
+        :param name: the name of the output column corresponding to this objective
+        """
         super().__init__(
-            class_name=class_name, frequency=frequency, func=func, **kwargs
+            class_name=class_name, frequency=frequency, func=func, name=name
         )
         self.key_value = key_value
         self._add_repr("key_value")
@@ -402,6 +490,7 @@ class VariableReader(EPReader):
     def _default_name(self):
         return self.variable_name
 
+    # FIXME: The return types for this function are inconsistent
     def __call__(self, results: _results_format) -> list:
         results_name = self.results_name()
         results_list = []
@@ -416,7 +505,7 @@ class VariableReader(EPReader):
                     meter_results = v
                     return self._process(meter_results)
             else:
-                if re.match(f"\w+\,{self.variable_name}", name):
+                if re.match(fr"\w+,{self.variable_name}", name):
                     meter_results = v
                     results_list.append(self._process(meter_results))
 
