@@ -1,27 +1,40 @@
+"""
+Classes used to represent the attributes of the building that can be varied,
+such as the thickness of the insulation, or the window to wall ratio.
+These parameters are separate from the value that they take on during any evaluation of the model.
+"""
 import warnings
-from typing import List, Dict, Tuple
 from abc import ABC, abstractmethod
+from typing import Callable, Dict, Iterable, List, Tuple
 
 import platypus
+from deprecated.sphinx import deprecated
 
-import eppySupport
-from IO_Objects import Descriptor, IOBase, Selector, DummySelector, AnyValue
-from config import range_parameter as conf
-import eppy_funcs as ef  # incorrectly detected as an unused import by pycharm
-from errors import ModeError
+import eppy_funcs as ef
 import pyehub_funcs as pf
+from IO_Objects import AnyValue, Descriptor, DummySelector, IOBase, Selector
+from config import range_parameter as conf
+from errors import ModeError
 
 
 # Descriptors
+from besos.types import PathLike
 
 
 class RangeParameter(Descriptor):
-    """Represents an aspect of a building that can take on value from an interval."""
+    """Represents a value that is contained in an interval.
+
+    .. warning:: RangeParameter is actually a kind of
+        :class:`Descriptor <IO_Objects.Descriptor>`,
+        not a :class:`Parameter`. This naming may be confusing.
+    """
 
     pandas_type = float
     rbf_type = "R"
 
-    def __init__(self, min_val=conf.get("min"), max_val=conf.get("max")):
+    def __init__(
+        self, min_val: float = conf.get("min"), max_val: float = conf.get("max")
+    ):
         """
 
         :param max_val: the minimum value
@@ -35,12 +48,17 @@ class RangeParameter(Descriptor):
         self._add_reprs(["min", "max"])
         self.platypus_type = platypus.Real(self.min, self.max)
 
-    def validate(self, val):
+    def validate(self, value: float) -> bool:
+        """Checks if `value` is contained within the range described by this Descriptor
+
+        :param value: The value to check.
+        :return: True if the value in the range, False otherwise.
+        """
         min_ = float("-inf") if self.min is None else self.min
         max_ = float("inf") if self.max is None else self.max
-        return min_ <= val <= max_
+        return min_ <= value <= max_
 
-    def sample(self, value):
+    def sample(self, value: float) -> float:
         """Transforms a value in [0, 1] into a value in [min, max].
         This transformation is uniform.
 
@@ -56,7 +74,7 @@ class RangeParameter(Descriptor):
 class DependentParameter(Descriptor):
     """ A parameter that the value of it depends on a range parameter"""
 
-    def __init__(self, index, factor, mode):
+    def __init__(self, index: int, factor, mode):
         """
 
         :param index: the index of the target range parameter in parameter list
@@ -108,7 +126,7 @@ class DependentParameter(Descriptor):
         return f"{self.__class__.__name__} index = {self.index}, mode = {self.mode}, factor = {self.factor}"
 
 
-class Category(platypus.Subset):
+class _Category(platypus.Subset):
     def __init__(self, elements):
         super().__init__(elements=elements, size=1)
 
@@ -120,19 +138,25 @@ class Category(platypus.Subset):
 
 
 class CategoryParameter(Descriptor):
-    """Represents a list of possible value of the parameter."""
+    """Represents value that is selected from a list of possible values.
+
+    .. warning:: CategoryParameter is actually a kind of
+        :class:`Descriptor <IO_Objects.Descriptor>`,
+        not a :class:`Parameter`.
+        This naming may be confusing.
+    """
 
     pandas_type = "category"
 
-    def __init__(self, options: list, **kwargs):
+    def __init__(self, options: Iterable, **kwargs):
         """
 
         :param options: a list of possible value this parameter can be set to
         """
         super().__init__(**kwargs)
-        self.options: list = options
-        self._to_repr["options"] = "options"
-        self.platypus_type = Category(self.options)
+        self.options = list(options)
+        self._add_repr("options")
+        self.platypus_type = _Category(self.options)
 
     def validate(self, value):
         return value in self.options
@@ -248,7 +272,9 @@ class FieldSelector(AbstractFieldSelector):
     """A selector that modifies one or more fields in an EnergyPlus building,
     based on the class, object and field names"""
 
-    def __init__(self, class_name=None, object_name=None, field_name=None):
+    def __init__(
+        self, class_name: str = None, object_name: str = None, field_name: str = None
+    ):
         """
 
         :param class_name: class of the object to modify
@@ -286,7 +312,7 @@ class FieldSelector(AbstractFieldSelector):
                 return [building.getobject(key=class_name, name=self.object_name)]
             if self.object_name:
                 # There should only one object matching the name, assuming the idf is valid
-                return [eppySupport.get_idfobject_from_name(building, self.object_name)]
+                return [ef.get_idfobject_from_name(building, self.object_name)]
             if class_name is not None:
                 # assume that we want the first object matching the key
                 # TODO: is this specific enough, or should we remove it? our JSON code does not support this
@@ -329,10 +355,51 @@ class FieldSelector(AbstractFieldSelector):
         raise ModeError(mode)
 
 
+class PathSelector(Selector):
+    """A Selector for modifying EnergyHub objects using a path."""
+
+    def __init__(self, parameter_path=None):
+        """
+        :param parameter_path: the path to the parameter to modify for the EnergyHub
+        """
+        super().__init__()
+        self.parameter_path = parameter_path
+
+    def get(self, hub):
+        return pf.get_by_path(hub.__dict__, self.parameter_path)
+
+    def set(self, hub, value):
+        pf.set_by_path(hub.__dict__, self.parameter_path, value)
+
+    def multiply(self, hub, value):
+        self.set(hub, value * self.get(hub))
+
+    def add(self, hub, value):
+        self.set(hub, value + self.get(hub))
+
+    def setup(self, hub) -> None:
+        """Modifies the building so that it is ready for this selector"""
+        pass
+
+
 class GenericSelector(Selector):
     """A selector that supports custom get/set functions"""
 
-    def __init__(self, set=None, get=None, setup=None):
+    def __init__(
+        self, set: Callable = None, get: Callable = None, setup: Callable = None
+    ):
+        """
+
+        :param set: The function to use when setting.
+            Must accept a building and a value. Can modify the building in any way.
+        :param get: The function to use when getting a value from a building.
+            Must accept a building, and should return a list of the current values
+            of the fields this selector affects.
+        :param setup: The function to use when setting up the building.
+            This function must accept a building. It may modify the building in any way.
+            This will be run once when the evaluator is initialized with a building.
+            (or when the building the evaluator uses is changed.)
+        """
         super().__init__()
         self._set = set
         self._get = get
@@ -381,19 +448,20 @@ class Parameter(IOBase):
         selector: Selector = None,
         value_descriptor: Descriptor = None,
         mode="Set",
-        **kwargs,
+        name="",
     ):
         """
 
         :param selector: a Selector describing how to modify the building
         :param value_descriptor: a Descriptor specifying which values to use
-        :param setup: a setup function to run on the building
         :param mode: the desired transformation function to use
             Set: set the sample as the value
             Multiply: multiply the pre-existing value by the sample factor
             Add: add the sample value onto the pre-existing value
+        :param name: the name of the Parameter. Used for readability
+            and column labelling.
         """
-        super().__init__(**kwargs)
+        super().__init__(name=name)
         self.selector = selector or DummySelector()
         self.value_descriptor = value_descriptor or AnyValue()
         assert mode in self.mode_options, f"Invalid mode option"
@@ -436,90 +504,70 @@ class Parameter(IOBase):
         return self.value_descriptor.platypus_type
 
 
-# TODO: Cleanup this mostly duplicated class by figuring out
-class ParameterEH(IOBase):
+@deprecated(
+    version="1.6.0",
+    reason=(
+        "Use a :class:`PathSelector` and a :class:`Parameter` instead. "
+        "These have been swapped in automatically."
+    ),
+)
+class ParameterEH:
+    """A parameter for use with EnergyHub"""
+
     mode_options = {"Set", "Multiply", "Add"}
 
-    def __init__(
-        self,
-        parameter_path=None,
+    # this method overrides the default behaviour when trying to create a new ParameterEH
+    # instead of creating a ParameterEH, it creates a PathSelector,
+    # puts it in a Parameter, and returns that parameter.
+    def __new__(
+        cls,
+        parameter_path: List[str] = None,
         value_descriptor: Descriptor = None,
-        mode="Set",
+        mode: str = "Set",
         **kwargs,
     ):
-        """
+        """Swaps out the ParameterEH for a Parameter and PathSelector for backwards compatibility.
 
         :param parameter_path: the path to the parameter to modify for the EnergyHub
         :param value_descriptor: a Descriptor specifying which values to use
         :param mode: the desired transformation function to use
-            Set: set the sample as the value
-            Multiply: multiply the pre-existing value by the sample factor
-            Add: add the sample value onto the pre-existing value
+          Set: set the sample as the value
+          Multiply: multiply the pre-existing value by the sample factor
+          Add: add the sample value onto the pre-existing value
         """
-        super().__init__(**kwargs)
-        self.parameter_path = parameter_path
-        self.value_descriptor = value_descriptor or AnyValue()
-        assert mode in self.mode_options, f"Invalid mode option"
-        self.mode = mode
-        self._add_reprs(["parameter_path", "value_descriptor"], True)
-
-    def transformation_function(self, hub, value) -> None:
-        """Mutates the building based on the value provided."""
-        if self.mode is "Multiply":
-            pf.set_by_path(
-                hub.__dict__,
-                self.parameter_path,
-                (value * pf.get_by_path(hub.__dict__, self.parameter_path)),
-            )
-        if self.mode is "Add":
-            pf.set_by_path(
-                hub.__dict__,
-                self.parameter_path,
-                (value + pf.get_by_path(hub.__dict__, self.parameter_path)),
-            )
-        if self.mode is "Set":
-            pf.set_by_path(hub.__dict__, self.parameter_path, value)
-
-    def sample(self, value: float):
-        """Takes a value in the range 0-1 and returns a valid value for this parameter"""
-        return self.value_descriptor.sample(value)
-
-    def validate(self, value):
-        """Checks if value is a valid value for this parameter.
-
-        :param value:
-        :return: True if the value is valid False otherwise
-        """
-        return self.value_descriptor.validate(value)
-
-    @property
-    def platypus_type(self):
-        """The platypus equivalent of this parameter"""
-        return self.value_descriptor.platypus_type
+        selector = PathSelector(parameter_path)
+        return Parameter(
+            selector=selector, value_descriptor=value_descriptor, mode=mode, **kwargs
+        )
 
 
 class wwrSelector(Selector):
-    """ selector for wwr """
+    """
+    :class:`Selector <IO_Objects.Selector>` for window to wall ratio.
+    """
 
     def __init__(self):
         super().__init__()
 
     def set(self, building, value):
-        """Sets the wwr in the building to the provided value
+        """Sets the window to wall ratio of the building to the provided value
 
         :param building: the building to modify
-        :param value: tha value of the wwr
+        :param value: the value of the wwr
         """
         ef.wwr_all(building, value)
 
-    def get(self):
+    def get(self, building):
         # This feature has not yet been requested
         raise NotImplementedError(
             "Calculation of window to wall ratio is not supported."
         )
 
-    def setup(self, building):
-        """A custom function for modifying the building so that it is ready for this selector
+    def setup(self, building) -> None:
+        """Adjusts the building to have at most one window per wall,
+        making it ready to have its window to wall ratio modified.
+
+        :param building: The building to adjust
         """
         ef.one_window(building)
 
@@ -573,16 +621,3 @@ def expand_plist(pList: keyFormat) -> List:
         for name, subProps in pList.items()
         for prop, (min_, max_) in subProps.items()
     ]
-
-
-if __name__ == "__main__":
-    pass
-    # a few simple tests of the inputs module
-    # edit: moved tests to test_parameter.py
-
-    # import eppy_funcs as ef
-
-    # transformation functions should modify the idf
-    # TODO: Test transformation functions
-
-    # print('tests passed')
