@@ -1,3 +1,8 @@
+"""
+Classes used to bundle the parameters, objectives and constraints,
+and to manage operations that involve all of them at once,
+such as converting data related to the problem to a DataFrame.
+"""
 import warnings
 from typing import Union, List, Dict, Callable, Iterable
 
@@ -15,7 +20,20 @@ import parameters
 # also consider storing the direction of optimisation inside the objectives
 # might be able to inherit some of the constraint parsing from platypus, not sure if that is worth the hassle
 class Problem(IO_Objects.ReprMixin):
-    """A class that collects all of the inputs, outputs and constraints related to a building."""
+    """
+    A class that collects all of the inputs, outputs and constraints related to a model.
+    Problems track what inputs are valid, and how to apply those inputs to a model.
+
+    It tracks constraint bounds.
+    Automatically converts certain shortcut notation:
+    - Strings become name only Parameters
+    - Integers become that many numbered Parameters
+    Gives access to names of all parts of the problem
+    Resolves duplicate names
+    Converts numpy arrays to a DataFrame matching the format of the problem or some
+    combination pieces of the format (ex: only inputs and constraints)
+    Can convert to a platypus Problem (which lacks an evaluation function).
+    """
 
     valid_parts: List[str] = ["inputs", "outputs", "constraints", "violation"]
     default_converters = {
@@ -41,11 +59,16 @@ class Problem(IO_Objects.ReprMixin):
         :param outputs: A list of Objectives, or an integer.
             If a list is used, strings are converted to Objectives and this list determines the valid inputs.
             If an integer, this problem requires that many outputs
-        :param constraints:
+        :param constraints: A list of Objectives to be used as constraints, or an integer.
+            If a list is used, strings are converted to Objectives and this list determines the valid inputs.
+            If an integer, this problem requires that many constraints.
         :param add_outputs: Outputs that don't need to be optimized in optimization algorithm
-        :param constraint_bounds:
-        :param minimize_outputs:
-        :param converters:
+        :param constraint_bounds: A list of strings that are parsable by platypus, and which represent bounds
+            on the corresponding constraints.
+        :param minimize_outputs: A list of booleans indicating if the corresponding output should be
+            minimized or maximized. (Default is to minimize all outputs).
+        :param converters: A dictionary with keys from {"outputs", "constraints"} where values indicate how to
+            convert those kinds of values to appropriate objectives/constraints for this problem.
         """
         super().__init__()
         self.converters = converters or self.default_converters
@@ -246,8 +269,7 @@ class Problem(IO_Objects.ReprMixin):
         return problem
 
     def pre_optimisation(self):
-        """Prepare optimisation for dependent parameter and non objectives.
-        """
+        """Prepare optimisation for dependent parameter and non objectives."""
         for i in range(self.num_inputs):
             if type(self.inputs[i].value_descriptor).__name__ == "DependentParameter":
                 self.depend_params[i] = []
@@ -255,8 +277,7 @@ class Problem(IO_Objects.ReprMixin):
             self.add_outputs_list = []
 
     def overwrite_values(self, values):
-        """Replace dataframe value with dependent parameter's value.
-        """
+        """Replace dataframe value with dependent parameter's value."""
         if not self.depend_params:
             return values
         l_values = list(values)
@@ -269,21 +290,18 @@ class Problem(IO_Objects.ReprMixin):
         return tuple(l_values)
 
     def get_depend_value(self, depend_param, values):
-        """Return value of dependent parameter
-        """
+        """Return value of dependent parameter"""
         return depend_param.get_value(values[depend_param.index])
 
     def record_results(self, inputs, results):
-        """Record add_outputs results
-        """
+        """Record add_outputs results"""
         inputs = list(inputs)
         for obj in self.add_outputs:
             inputs.append(obj(results))
         self.add_outputs_list.append(inputs)
 
     def get_non_objective(self, df):
-        """Create a list for dataframe with add_outputs value
-        """
+        """Create a list for dataframe with add_outputs value"""
         l = []
         for row in df.values.tolist():
             for obj in self.add_outputs_list:
@@ -298,8 +316,7 @@ class Problem(IO_Objects.ReprMixin):
         return l
 
     def overwrite_df(self, df):
-        """Insert dependent parameter's value and/or add_outputs' value to dataframe
-        """
+        """Insert dependent parameter's value and/or add_outputs' value to dataframe"""
         labels = df.columns.values
         for index in self.depend_params:
             df.drop(columns=labels[index], inplace=True)
@@ -308,7 +325,9 @@ class Problem(IO_Objects.ReprMixin):
             l = self.get_non_objective(df)
             for i, data in enumerate(l):
                 df.insert(
-                    len(df.columns), self.add_outputs[i].name, data,
+                    len(df.columns),
+                    self.add_outputs[i].name,
+                    data,
                 )
         return df
 
@@ -330,7 +349,14 @@ class Problem(IO_Objects.ReprMixin):
 
 # TODO: consider having shortcuts for the converters instead of making this a whole different class
 class EPProblem(Problem):
-    """A problem with defaults that are appropriate for EnergyPlus simulations"""
+    """
+    A problem with defaults that are appropriate for EnergyPlus simulations
+
+    Strings for objectives/constraints become a MeterReader for the meter with
+    that name.
+
+    Integers still become numbered Parameters.
+    """
 
     default_converters = {
         "outputs": objectives.MeterReader,
