@@ -1,3 +1,7 @@
+"""
+Various ways to generate datapoints from the solution space, such that it is well covered.
+Uses :class:`IO_Objects.Descriptor` to determine which inputs are valid.
+"""
 from typing import Callable, Any
 
 import numpy as np
@@ -45,18 +49,28 @@ def dist_sampler(
     return df
 
 
-def add_extremes(df, problem):
-    """adds two datapoints with the minimum and maximum values for each attribute
+def add_extremes(df: pd.DataFrame, problem: Problem) -> pd.DataFrame:
+    """Adds two datapoints with the minimum and maximum values for each attribute.
 
-    :param data:
-    :param problem:
-    :return:
+    :param df: existing data to which the extreme values should be added.
+    :param problem: the problem defining which values are valid.
+    :return: df with 2 added rows, one with maximal values, one with minimal values.
     """
     new = dist_sampler(extremes, problem, 2)
     return pd.concat([df, new], ignore_index=True)
 
 
 def extremes(samples: int, attributes: int = None) -> np.ndarray:
+    """This sampler generates datapoints corresponding to the extremes of a problem.
+
+    :param samples: the number of samples. Must be equal to 2
+        (this argument is available in order to implement the sampler interface)
+        If only one argument is provided, that argument will be assigned to attributes,
+        even though samples is listed first.
+    :param attributes: the number of attributes for which to generate samples
+    :return: An array containing 2 samples: one with all zeroes, one with all ones.
+        When transformed by a problem, these become minimal and maximal values for that problem.
+    """
     if attributes is None:
         samples, attributes = 2, samples
     assert samples == 2, "extremes can only produce two samples"
@@ -89,7 +103,7 @@ def lhs(samples: int, attributes: int, *args, **kwargs) -> np.ndarray:
 
 
 def full_factorial(samples: int, attributes: int, level: int = None) -> np.ndarray:
-    if level == None:
+    if level is None:
         level = np.int(np.exp(np.log(samples) / attributes))
         if (level ** attributes) > level:
             print(
@@ -224,7 +238,11 @@ def MC_Voronoi_estimate(P, problem):
     # P are the existing points
 
     # initialize arrays
-    V_P = np.zeros([len(P),])  # Vector of all Voronoi sample estimates
+    V_P = np.zeros(
+        [
+            len(P),
+        ]
+    )  # Vector of all Voronoi sample estimates
     # per existing point collect randomly selected 100 points
 
     S = dist_sampler(
@@ -251,7 +269,11 @@ def LOLA_estimate(N_P, P, model, scaler, scaler_out):
 
     n = len(P)
     ind = 0
-    E = np.empty([n,])
+    E = np.empty(
+        [
+            n,
+        ]
+    )
     for p_r in P:
         grad = Gradient_estimation(
             N_P[:, :, ind], p_r, model, scaler=scaler, scaler_out=scaler_out
@@ -284,7 +306,11 @@ def update_Neighborhood(N_P, P, S_P, P_new):
         for p_r in P:
             if sum(p_r == p_new) < len(P[0, :]):
                 N_temp = np.dstack([N_P[:, :, ind]] * m)
-                S_temp = np.zeros([m,])
+                S_temp = np.zeros(
+                    [
+                        m,
+                    ]
+                )
                 for i in range(m):
                     # replace one sample with the new one
                     if sum(sum(N_temp[:, :, i] == p_new)) < len(P[0, :]):
@@ -317,7 +343,11 @@ def init_Neighborhood(P, P_new=None, verbose=False):
         # case I: first initialization after static sampling
         n = len(P)  # number of samples
         N_P = np.empty([m, d, n])
-        S_P = np.empty([n,])
+        S_P = np.empty(
+            [
+                n,
+            ]
+        )
         P_ref = (
             P * 1.0
         )  # array of all samples to be considered as reference in the next for loop
@@ -331,7 +361,11 @@ def init_Neighborhood(P, P_new=None, verbose=False):
             P_ref = P_new * 1.0
         n_new = len(P_ref)  # number of new samples
         N_P = np.empty([m, d, n_new])
-        S_P = np.empty([n_new,])
+        S_P = np.empty(
+            [
+                n_new,
+            ]
+        )
 
     for i in range(len(P_ref)):
         mask = P != P_ref[i, :]
@@ -355,8 +389,16 @@ def Neighborhood_score(N, p_r):
     m = len(N)
 
     # initialize arrays
-    Dist_cand = np.empty([m,])
-    minDist = np.empty([m,])
+    Dist_cand = np.empty(
+        [
+            m,
+        ]
+    )
+    minDist = np.empty(
+        [
+            m,
+        ]
+    )
 
     # compute distance matrix
     C = 0
@@ -382,7 +424,11 @@ def Gradient_estimation(N, p_r, model, scaler, scaler_out):
     d = len(p_r)
 
     P_mat = np.empty([m, d])
-    F_mat = np.empty([m,])
+    F_mat = np.empty(
+        [
+            m,
+        ]
+    )
 
     for i in range(m):
         P_mat[i, :] = N[i, :] - p_r
