@@ -1,11 +1,11 @@
 import pytest
 from pyKriging.krige import kriging
+import pyKriging
 
-import eppy_funcs as ef
 import sampling
-from evaluator import EvaluatorEP, AdaptiveSR
-from parameters import expand_plist, wwr
-from problem import EPProblem
+from besos.evaluator import AdaptiveSR, EvaluatorGeneric
+from besos.parameters import Parameter, RangeParameter
+from besos.problem import Problem
 
 
 # Class taken directly from the notebook
@@ -52,26 +52,26 @@ class KirgingEval(AdaptiveSR):
 
 
 @pytest.fixture
-def building():
-    return ef.get_building()
+def problem():
+    return Problem(
+        [
+            Parameter(value_descriptor=RangeParameter(0, 1), name="first"),
+            Parameter(value_descriptor=RangeParameter(0, 1), name="second"),
+        ],
+        outputs=1,
+    )
 
 
 @pytest.fixture
-def parameters():
-    parameters = expand_plist(
-        {"Mass NonRes Wall Insulation": {"Thickness": (0.05, 0.99)}}
-    )
-
-    parameters.append(wwr())
-    return parameters
+def evaluation_function():
+    return pyKriging.testfunctions().branin
 
 
 @pytest.mark.slow
-def test_init(building, parameters):
+def test_init(problem, evaluation_function):
     """Testing the initialization and functionality of an adaptive surrogate model"""
 
-    problem = EPProblem(parameters)
-    evaluator = EvaluatorEP(problem, building)
+    evaluator = EvaluatorGeneric(evaluation_func=evaluation_function, problem=problem)
 
     inputs = sampling.dist_sampler(sampling.seeded_sampler, problem, 10)
     inputs = sampling.add_extremes(inputs, problem)
