@@ -1,22 +1,25 @@
+import shutil
+
+import numpy as np
 import pytest
 
+from evaluator import EvaluatorGeneric
 from optimizer import rbf_opt
-from evaluator import EvaluatorSR
 from parameters import Parameter, RangeParameter
 from problem import Problem
-import numpy as np
 
 
+@pytest.mark.skipif(shutil.which("bonmin") is None, reason="Requires bonmin to run.")
 def test_rbfopt():
-    """Simple test to make sure RBFopt can be run properly"""
+    """Simple test to make sure RBFOpt can be run properly"""
 
-    def obj_funct(x):
-        return (x[0] * x[1] - x[2],), ()
+    def objective_function(x):
+        return (x[0] * x[1] - x[2],)
 
     param_list = [Parameter(value_descriptor=RangeParameter(0, 10)) for _ in range(3)]
     problem = Problem(param_list, 1)
 
-    evaluator = EvaluatorSR(obj_funct, problem, error_mode="Silent")
+    evaluator = EvaluatorGeneric(objective_function, problem, error_mode="Silent")
 
     opt = rbf_opt(evaluator, 30)
     value = opt.iloc[0]
